/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit

enum DocumentError : Error {
  case runtimeError(String)
}

class Note: UIDocument {

  // MARK: - Static-Properties
  static let fileExtension = "txt"

  // MARK: - Properties
  var documentText: String?
  var title: String!

  // MARK: - Overridden Instance Methods
  override func load(fromContents contents: Any, ofType typeName: String?) throws {
    guard let data = contents as? Data , data.count > 0 else { return }

    documentText = String(data: data, encoding: .utf8)
  }
  
  override func contents(forType typeName: String) throws -> Any {
    documentText = documentText ?? ""

    guard let documentData = documentText?.data(using: .utf8) else {
      throw DocumentError.runtimeError("Unable to convert String to data")
    }

    return documentData
  }
}

// MARK: - Static-Methods
extension Note {

  // Creates a note with a title/filename
  static func createNote(_ noteTitle: String) -> Note? {
    guard let fileURL = fileUrlForDocumentNamed(noteTitle) else { return nil }

    let noteDocument = Note(fileURL: fileURL)
    noteDocument.title = noteTitle

    return noteDocument
  }

  // Given an array of filenames, return an array of notes from the file system
  static func arrayOfNotesFromArrayOfFileNames(_ fileNames: [String]) -> [Note] {
    var notes: [Note] = []
    
    for fileName in fileNames {
      let noteTitle = fileName.replacingOccurrences(of: ".txt", with: "")
      
      guard let note = createNote(noteTitle) else { continue }
      notes.append(note)
    }
    return notes
  }
  
  // Returns all notes in file system
  static func getAllNotesInFileSystem() -> [Note] {
    guard let localDocumentsDirectory = localDocumentsDirectoryURL() else { return [] }

    let localDocumentsDirectoryPath = localDocumentsDirectory.path
    let localDocuments: [String]?
    do {
      localDocuments = try FileManager.default.contentsOfDirectory(atPath: localDocumentsDirectoryPath)
    } catch _ {
      print("error accessing contents from directory")
      localDocuments = nil
    }
    
    guard let fileNames = localDocuments else { return [] }

    return arrayOfNotesFromArrayOfFileNames(fileNames)
  }
  
  // Returns the file URL for the file to be saved at
  static func fileUrlForDocumentNamed(_ name: String) -> URL? {
    guard let baseURL = localDocumentsDirectoryURL() else { return nil }

    let protectedName: String
    if name.isEmpty {
      protectedName = "Untitled"
    } else {
      protectedName = name
    }

    return baseURL.appendingPathComponent(protectedName)
      .appendingPathExtension(fileExtension)
  }
  
  
  static func localDocumentsDirectoryURL() -> URL? {
    guard let documentPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first else { return nil }
    
    let localDocumentsDirectoryURL = URL(fileURLWithPath: documentPath)
    return localDocumentsDirectoryURL
  }
}


